gem 'activesupport'
require 'active_support/all'

module NginxController
  module Acme
    def self.acme_contact
      ENV['ACME_CONTACT'].presence || 'domini@intercom.it'
    end

    def self.acme_endpoint
      ENV['ACME_ENDPOINT'].presence || 'https://acme-v02.api.letsencrypt.org/directory'
    end

    def self.le_root
      ENV['LE_SSL_ROOT'].presence || File.join(NginxController.ssl_root, 'letsencrypt')
    end

    def self.get(options)
      Ruby.new(options)
    end    

    class Base
      attr_reader :options

      def initialize(options)
        @options = options
      end

      def valid?
        raise 'unimplemented'
      end

      def expires_at
        raise 'unimplemented'
      end

      def privkey_path
        live_privkey_path
      end
  
      def cert_path
        live_fullchain_path
      end 

      def names
        raise 'unimplemented'
      end

      def prepare!
        raise 'unimplemented'
      end

      def request_certificate!
        raise 'unimplemented'
      end

      def live_cert_dir
        raise 'unimplemented'
      end

      def live_privkey_path
        File.join(live_cert_dir, 'key.pem')
      end

      def live_chain_path
        File.join(live_cert_dir, 'chain.pem')
      end

      def live_cert_path
        File.join(live_cert_dir, 'cert.pem')
      end

      def live_fullchain_path
        File.join(live_cert_dir, 'full_chain.pem')
      end

    end
  end
end
