require 'proxy/provision/acme/base'
require 'open3'

module NginxController
  module Acme
    class Certbot < Proxy::Provision::Acme::Base
      def initialize(service)
        @service = service
      end

      def prepare!
        unless File::exists?(@service.le_cert_dir)
          FileUtils.mkdir_p @service.le_cert_dir
        end
        shared_accntdir = File::join(File::dirname(@service.le_cert_dir), '__accounts')
        unless File::exists?(shared_accntdir)
          FileUtils::mkdir_p(shared_accntdir)
        end
        accntdir = File::join(@service.le_cert_dir, 'accounts')
        unless File::symlink?(accntdir)
          if File::exists?(accntdir)
            bckaccntdir = File::join(@service.le_cert_dir, 'accounts.orig')
            File::rename(accntdir, bckaccntdir)
          end
          File::symlink(shared_accntdir, accntdir)
        end
      end

      def request_certificate!
        #certbot-auto -v --config-dir /etc/letsencrypt/dom1 -m angelo@intercom.it --agree-tos --test-cert certonly --webroot -w /srv/http-proxy/dom1/ -d domain1.intercom.it

        options = [
          "--agree-tos",
          "-n",
          "--no-eff-email",
          "--config-dir #{@service.le_cert_dir}",
          "-m #{Proxy::Provision.config.acme_contact}",
          "certonly"
        ]
        if @service.use_dns01?
          options += [
            "--dns-rfc2136",
            "--dns-rfc2136-credentials #{@service.dnssec_key}"
          ]
        else
          options += [
            "--webroot",
            "-w #{@service.le_web_root}",
          ]
        end

        options << "-d #{@service.server_name}"
        options += @service.aliases.compact.map{|a| "-d #{a}"} if @service.aliases && @service.aliases.length > 0

        options.unshift('--test-cert') if Proxy::Provision.config.acme_test

        dir = Proxy::Provision.config.acme_endpoint.strip
        dir += '/' unless dir =~ /\/\Z/
        dir += 'directory'

        options.unshift("--server #{dir}") if Proxy::Provision.config.acme_endpoint

        cmd = "#{Proxy::Provision.config.certbot_path} #{options.join(' ')}"

        puts "  - Build certificate for #{[@service.server_name, @service.aliases].flatten.compact.join(', ')}"

        begin
          stdout, stderr, status = Open3.capture3(cmd)
          puts "    " + stdout.lines.map(&:strip).join("\n    ")
        rescue
          stderr = $!.to_s
          status = false
        end

        unless status && status.success?
          puts "   ! Cannot generate SSL cert"
          puts "   ! " + stderr.lines.map(&:strip).join("\n   ! ")
        end

        status
      end

      def live_cert_dir
        File::join(@service.le_cert_dir, 'live', @service.server_name)
      end

      def live_privkey
        File::join(live_cert_dir, 'privkey.pem')
      end

      def live_chain
        File.join(live_cert_dir, 'chain.pem')
      end

      def live_cert
        File.join(live_cert_dir, 'cert.pem')
      end

      def live_fullchain
        File::join(live_cert_dir, 'fullchain.pem')
      end
    end
  end
end
