require 'eventmachine'
require 'http/parser'
require 'uri'
require 'json'

module NginxController
  module Docker
    class Query < EM::Connection
      include EventMachine::Deferrable
      attr_accessor :query, :params

      QUERIES = {
        services: '/services',
        tasks: '/tasks'
      }

      def self.exec(query, params = {})
        EventMachine.connect_unix_domain('/var/run/docker.sock', self) do | q |
          q.query = query
          q.params = params
        end
      end

      def initialize
        @parser = Http::Parser.new
        @body = ''

        @parser.on_body = proc do |chunk|
          @body << chunk
        end

        @parser.on_message_complete = proc do |env|
          set_deferred_status :succeeded, JSON.parse(@body)
          close_connection
        end
      end

      def query=(q)
        raise Exception.new("unknown query #{q}") unless QUERIES.keys.include?(q.to_sym)
        @query = q.to_sym
      end

      def encoded_params
        (@params || {}).map do | k, v |
          "#{k}=#{v.to_json}"
        end.join('&')
      end

      def connection_completed
        uri = "/v1.39#{QUERIES[self.query]}?#{self.encoded_params}"
        req = []
        req << "GET #{URI.encode(uri)} HTTP/1.1"
        req << "Host: docker"
        req << "Accept: */*"
        req << "User-Agent: nginx controller"
        req << ""
        req << ""
        #puts req.join("\r\n")
        send_data req.join("\r\n")
      end

      def receive_data(response)
        @parser << response
      end
    end
  end
end