require 'nginx_controller/base_service'
require 'fileutils'

module NginxController
  class MasterService < BaseService

    def initialize
    end

    def id
      '000000000000'
    end

    def name
      '000default'
    end
    
    def managed?
      true
    end

    def running?
      true
    end

    def settled?
      true
    end

    def ==(other)
      other.is_a?(MasterService)
    end

    def ===(other)
      other.is_a?(MasterService) &&
        self.ssl_certificate == other.ssl_certificate &&
        self.ssl_certificate_key == other.ssl_certificate_key
    end

    def <=>(other)
      self.name <=> other.name
    end

    def diff(other)
      {}
    end

    def configure!
      create_directories
      super
    end

    def reconfigure!
      configure!
    end

    def deconfigure!
      if File.exists?(config_filename)
        FileUtils.rm(config_filename)
      end
    end
    
    def ssl_certificate
      if acme.valid?
        acme.cert_path
      else
        self_signed_cert.cert_path
      end
    end

    def ssl_certificate_key
      if acme.valid?
        acme.privkey_path
      else
        self_signed_cert.privkey_path
      end
    end

    def server_names
      [ENV['DEFAULT_HOST']]
    end

    def validation_host
      ENV['VALIDATION_HOST']
    end

    def request_letsencrypt?
      !acme.valid? || acme.expiring?
    end

    def letsencrypt?
      true
    end

    protected

    def create_directories
      ['/storage/logs/validation', '/storage/logs/default', '/srv/validation/well-known'].each do | dir |
        FileUtils.mkdir_p(dir)
      end
    end
    
    def self_signed_cert
      setup_self_signed_cert unless @self_signed_cert
      @self_signed_cert
    end

    def setup_self_signed_cert
      @self_signed_cert ||= NginxController::SelfSignedCert.new({
        names: server_names,
        service_name: 'server'
      })
      @self_signed_cert.create! unless @self_signed_cert.valid?
    end

    def acme
      @acme ||= NginxController::Acme.get({
        names: server_names,
        service_name: 'server'
      })
    end

    def render(template = 'default', locals = nil)
      @locals = locals
      tpl = File.join(File.dirname(__FILE__), 'templates', "#{template}.erb")
      raise StandardError.new "Template #{template} not found" unless File::exists?(tpl)
      erb = ERB.new(File.read(tpl), nil, '-')
      erb.result self.get_binding
    end

    def get_binding
      binding
    end
  end
end